#ifndef LINKED_LIST_H
#define LINKED_LIST_H

#include <stdbool.h>

/** 
 * @brief Data structure holding an element of a \a LinkedList
 */
typedef struct elt {
  int val;
  struct elt *next;
} Element;

/** 
 * @brief Data structure holding a \a LinkedList
 */
typedef struct {
  Element *first;
} LinkedList;

/**
 * @brief Creates a new \a LinkedList
 * @return The pointer on the created \a LinkedList
 */
LinkedList *newLL();

/**
 * @brief Displays on terminal the values contained in a \a LinkedList
 * @param[in] l the \a LinkedList to dump
 */
void dumpLL(LinkedList *l);

/**
 * @brief Computes the number of elements in the list
 * @param[in] l the concerned \a LinkedList
 * @return Number of elements in the list (0 if empty)
 */
int sizeLL(LinkedList *l);

/**
 * @brief Frees a \a LinkedList
 * @param[in] l the \a LinkedList to free
 */
void freeLL(LinkedList *l);

/**
 * @brief Inserts a value in a \a LinkedList
 * @param[in] l the concerned \a LinkedList
 * @param[in] v the value to insert
 */
void addLL(LinkedList *l, int v);

/**
 * @brief Removes a value from a \a LinkedList
 * @param[in] l the concerned \a LinkedList
 * @param[in] v the value to remove
 */
void removeLL(LinkedList *l, int v);

#endif /* LINKED_LIST_H */
